@@ -178,7 +178,11 @@ and defaults to the product of the horizontal and vertical
 dimensions. B<Note> that I am B<strongly> considering eliminating this
 value, since it appears to me that any value other than the default
 results in an impossible puzzle. As of version 0.005_01, specification
-of the third value is deprecated. 
+of the third value is deprecated. At the first release after September 1
+2016 a warning will be displayed the first time three values are
+supplied. After a further six months this will progress to a warning
+every time three values are supplied. Finally, supplying three values
+will become a fatal error.
 
 Setting this attribute modifies the following "real" attributes:
 
@@ -368,17 +372,12 @@ This "virtual" attribute is a convenience, which causes the object to
 be configured with the given number of cells, but no topology. The
 topology must be added later using the add_set method once for each
 set of cells to be created.
+
 The value must be either a comma-separated list of one to three numbers
 (e.g. '81,9,9') or a reference to a list containing one to three
 numbers (e.g. [81, 9, 9]). The first (and only required) number gives the
 number of cells. The second, if supplied, sets the 'columns' attribute,
-and the third, if supplied, sets the 'rows' attribute.
-
-value set must be either a comma-separated list of three numbers (e.g.
-'3,2,6') or a reference to a list containing three numbers (e.g. [3, 2,
-6]). Either way, the numbers represent the horizontal dimension of the
-rectangle (in columns), the vertical dimension of the rectangle (in
-rows), and the overall size of the puzzle square. For example,
+and the third, if supplied, sets the 'rows' attribute. For example,
 
  $su->set (null => [36, 6]);
  $su->add_set (r0 => 0, 1, 2, 3, 4, 5);
@@ -393,7 +392,7 @@ rows), and the overall size of the puzzle square. For example,
 
 Generates the topology equivalent to
 
- $su->set (brick => [3, 2, 6])
+ $su->set (brick => [3, 2])
 
 =item output_delimiter (string)
 
@@ -585,7 +584,7 @@ use warnings;
 
 use base qw{Exporter};
 
-our $VERSION = '0.015_01';
+our $VERSION = '0.017_01';
 our @EXPORT_OK = qw{
     SUDOKU_SUCCESS
     SUDOKU_NO_SOLUTION
@@ -713,7 +712,7 @@ version 0.001
 =cut
 
 sub constraints_used {
-    my $self = shift;
+    my ( $self ) = @_;
     return unless $self->{constraints_used} && defined wantarray;
     return %{$self->{constraints_used}} if wantarray;
     my $rslt = join ' ', grep {
@@ -727,74 +726,29 @@ sub constraints_used {
 This method copies the current problem to the clipboard. If solution()
 has been called, the current solution goes on the clipboard.
 
-See L<CLIPBOARD SUPPORT> for what is needed for this to work.
+See L<CLIPBOARD SUPPORT|/CLIPBOARD SUPPORT> for what is needed for this
+to work.
 
 =cut
 
 {	# Local symbol block.
     my $copier;
     sub copy {
-	my $self = shift;
-	$copier ||= $^O eq 'MSWin32' ? (_copier_win32 () ||
-		croak <<eod) :
-Error - Copy to clipboard unavailable. Can not load Win32::Clipboard.
-eod
-	    $^O eq 'cygwin' ? (_copier_win32 () || _copier_xclip () ||
-		croak <<eod) :
-Error - Copy to clipboard unavailable. Can not load Win32::Clipboard,
-        and xclip has not been installed. For xclip, see
-	http://freshmeat.net/projects/xclip
-eod
-	    $^O eq 'darwin' ? (_copier_pbcopy () || _copier_xclip () ||
-		croak <<eod) :
-Error - Copy to clipboard unavailable. Can not find the pbcopy program,
-        which is supposed to come with Mac OS X. Can not find xclip
-	either. For xclip, see http://freshmeat.net/projects/xclip.
-eod
-	    $^O eq 'MacOS' ? croak <<eod :
-Error - Copy to clipboard unavailable under Mac OS 9 or below.
-eod
-	    _copier_xclip () || croak <<eod;
-Error - Copy to clipboard unavailable. The xclip program has not been
-        installed. See http://freshmeat.net/projects/xclip for a copy.
-eod
-	$copier->($self->_unload ());
+	my ( $self ) = @_;
+	( $copier ||= eval {
+		require Clipboard;
+		Clipboard->import();
+		sub {
+		    Clipboard->copy( join '', @_ );
+		    return;
+		};
+	    }
+	) or croak 'copy() unavailable; can not load Clipboard';
+	$copier->( $self->_unload() );
 	return $self;
     }
 }
 
-sub _copier_external {
-    my ($code, $probe) = @_;
-    `$probe`;	# Don't care what it returns.
-    return $? ? undef : sub {
-	open my $hdl, '|-', $code
-	    or croak "Error - failed to open output handle to $code: $!";
-	print $hdl @_;
-	close $hdl;
-	return '';
-    }
-}
-
-sub _copier_pbcopy {
-    return _copier_external (pbcopy => 'pbcopy -help 2>&1');
-}
-
-sub _copier_xclip {
-    return _copier_external (xclip => 'xclip -o');
-}
-
-sub _copier_win32 {
-    eval {
-	_require( 'Win32::Clipboard' );
-	1;
-    } and return sub {
-	(my $s = join '', @_) =~ s/\n/\r\n/mg;
-	Win32::Clipboard->new ()->Set ($s);
-    };
-    return;
-}
-
-
 =item $su->drop_set ($name)
 
 This method removes from the current topology the set with the given
@@ -851,7 +805,7 @@ freedom and the observation that about a third of the cells are given
 in a typical Sudoku puzzle. My experience with the default is:
 
  topology        comment
- brick 3,2,6     default is OK
+ brick 3,2       default is OK
  corresponding 3 default is OK
  cube 3          default is too large
  cube half       default is OK
@@ -904,14 +858,14 @@ one solution.
 =cut
 
 sub generate {
-    my $self = shift;
+    my ( $self, $min, $max, $const ) = @_;
     my $size = @{$self->{cell}} - $self->{cells_unused};
-    my $min = shift || do {
-	floor ($size * $size /
-	    ($self->{largest_set} * keys %{$self->{set}}));
+    $min ||= do {
+	floor( $size * $size /
+	    ( $self->{largest_set} * keys %{ $self->{set} } ) );
     };
-    my $max = shift || floor ($min * 1.5);
-    my $const = shift || 'F N B T';
+    $max ||= floor( $min * 1.5 );
+    $const ||= 'F N B T';
     croak <<eod if ref $const && ref $const ne 'HASH';
 Error - The constraints argument must be a string or a hash reference,
     not a @{[ref $const]} reference.
@@ -1013,7 +967,7 @@ eod
 }
 
 sub _get_allowed_symbols {
-    my $self = shift;
+    my ( $self ) = @_;
     my $rslt = '';
     my $syms = @{$self->{symbol_list}};
     foreach (sort keys %{$self->{allowed_symbols}}) {
@@ -1028,12 +982,12 @@ sub _get_allowed_symbols {
 }
 
 sub _get_symbols {
-    my $self = shift;
+    my ( $self ) = @_;
     return join ' ', @{$self->{symbol_list}};
 }
 
 sub _get_topology {
-    my $self = shift;
+    my ( $self ) = @_;
     my $rslt = '';
     my $col = $self->{columns};
     my $row = $self->{rows} ||= floor (@{$self->{cell}} / $col);
@@ -1062,7 +1016,8 @@ sub _get_value {return $_[0]->{$_[1]}}
 
 This method pastes a problem from the clipboard.
 
-See L<CLIPBOARD SUPPORT> for what is needed for this to work.
+See L<CLIPBOARD SUPPORT|/CLIPBOARD SUPPORT> for what is needed for this
+to work.
 
 =cut
 
@@ -1070,68 +1025,24 @@ See L<CLIPBOARD SUPPORT> for what is needed for this to work.
 
     my $paster;
     sub paste {
-	my $self = shift;
-	$paster ||=  $^O eq 'MSWin32' ? (_paster_win32 () ||
-		croak <<eod) :
-Error - Paste from clipboard unavailable. Can not load Win32::Clipboard.
-eod
-	    $^O eq 'cygwin' ? (_paster_win32 () || _paster_xclip () ||
-		croak <<eod) :
-Error - Paste from clipboard unavailable. Can not load Win32::Clipboard,
-        and xclip has not been installed. For xclip, see
-	http://freshmeat.net/projects/xclip
-eod
-	    $^O eq 'darwin' ? (_paster_pbpaste () || _paster_xclip () ||
-		croak <<eod) :
-Error - Paste from clipboard unavailable. Can not find the pbpaste
-        program, which is supposed to come with Mac OS X. Can not find
-	xclip either. For xclip, see http://freshmeat.net/projects/xclip.
-eod
-	    $^O eq 'MacOS' ? croak <<eod :
-Error - Paste from clipboard unavailable under Mac OS 9 or below.
-eod
-	    (_paster_xclip () || croak <<eod);
-Error - Paste from clipboard unavailable. The xclip program has not been
-        installed. See http://freshmeat.net/projects/xclip for a copy.
-eod
-	$self->problem ($paster->());
-	$self->_unload ();
+	my ( $self ) = @_;
+	( $paster ||= eval {
+		require Clipboard;
+		Clipboard->import();
+		return sub {
+		    return Clipboard->paste();
+		};
+	    }
+	) or croak 'paste() unavailable; can not load Clipboard';
+
+	$self->problem( $paster->() );
+	$self->_unload();
 	return $self;
     }
 
 
 }	#	End local symbol block
 
-sub _paster_external {
-    my ($code, $probe) = @_;
-    `$probe`;	# Not interested in what probe returns.
-    return $? ? undef : sub {
-	local $/ = undef;
-	open my $hdl, '-|', $code
-	    or croak "Error - failed to open input handle from $code: $!";
-	my $buffer = <$hdl>;
-	close $hdl;
-	return $buffer;
-    }
-}
-
-sub _paster_pbpaste {
-    return _paster_external (pbpaste => 'pbpaste -help 2>&1');
-}
-
-sub _paster_xclip {
-    return _copier_external ('xclip -o' => 'xclip -o');
-}
-
-sub _paster_win32 {
-    eval {
-	_require( 'Win32::Clipboard' );
-	1;
-    } and return sub { return Win32::Clipboard->new()->Get(); };
-    return;
-}
-
-
 =item $su->problem ($string);
 
 This method specifies the problem to be solved, and sets the object
@@ -1161,8 +1072,8 @@ time you set the symbol names.
 =cut
 
 sub problem {
-    my $self = shift;
-    my $val = shift || '';
+    my ( $self, $val ) = @_;
+    $val ||= '';
     $val =~ m/\S/ or
 	$val = "$self->{symbol_list}[0] " x
 	(scalar @{$self->{cell}} - $self->{cells_unused});
@@ -1262,32 +1173,6 @@ eod
     return $self;
 }
 
-{
-
-    my %required;
-
-    sub _require {
-	my ( $module ) = @_;
-	$required{$module}
-	    and return $required{$module}->();
-	( my $fn = $module ) =~ s{ :: }{/}smxg;
-	$fn .= '.pm';
-	eval {
-	    my $rslt = require $fn;
-	    $required{$module} = sub { return $rslt };
-	    1;
-	} or do {
-	    my $err = $@;
-	    $required{$module} = sub { die $err };
-	};
-
-	return $required{$module}->();
-
-    }
-}
-
-
-
 my %mutator = (
     allowed_symbols => \&_set_allowed_symbols,
     autocopy => \&_set_value,
@@ -1331,19 +1216,20 @@ The object itself is returned.
 =cut
 
 sub set {
-    my $self = shift;
-    while (@_) {
-	my $name = shift;
+    my ( $self, @args ) = @_;
+    while ( @args ) {
+	my ( $name, $val ) = splice @args, 0, 2;
 	exists $mutator{$name} or croak <<eod;
 Error - Attribute $name does not exist, or is read-only.
 eod
-	$mutator{$name}->($self, $name, shift);
+	$mutator{$name}->($self, $name, $val );
     }
     return $self;
 }
 
 sub _set_allowed_symbols {
-    my ($self, $name, $value) = @_;
+##  my ( $self, $name, $value ) = @_;
+    my ( $self, undef, $value ) = @_;	# Name unused
     defined $value or $value = '';
     my $maxlen = 0;
     $self->{debug} and print <<eod;
@@ -1381,8 +1267,10 @@ eod
 }
 
 sub _set_brick {
-    my ($self, $name, $value) = @_;
+    my ( $self, undef, $value ) = @_;	# $name unused
     my ($horiz, $vert, $size) = ref $value ? @$value : split ',', $value;
+    defined $size
+	and $self->_deprecation_notice( 'brick_third_argument' );
     $size ||= $horiz * $vert;
     ($size % $horiz || $size % $vert) and croak <<eod;
 Error - The puzzle size $size must be a multiple of both the horizontal
@@ -1405,7 +1293,8 @@ eod
 }
 
 sub _set_corresponding {
-    my ($self, $name, $order) = @_;
+##  my ( $self, $name, $order ) = @_;
+    my ( $self, undef, $order ) = @_;	# Name unused
     my $size = $order * $order;
     $self->set (sudoku => $order);
     my $order_minus_1 = $order - 1;
@@ -1463,7 +1352,8 @@ eod
 );
 
 sub _set_cube {
-    my ($self, $name, $type) = @_;
+##  my ( $self, $name, $type ) = @_;
+    my ( $self, undef, $type ) = @_;	# Name unused
     if ($type =~ m/\D/) {
 	$cube{$type} or croak <<eod;
 Error - Cube type '$type' is not defined. Legal values are numeric (for
@@ -1490,14 +1380,15 @@ eod
 }
 
 sub _cube_set_names {
-    my ($order, $name, $x, $y, $z) = @_;
+    my ( $order, $name, $x, $y, $z ) = @_;
     my $tplt = sprintf '%s%d%%s%%d', $name, $x;
     return map {sprintf $tplt, @$_} [r => $y], [c => $z],
 	[s => floor ($y / $order) * $order + floor ($z / $order)]
 }
 
 sub _set_latin {
-    my ($self, $name, $size) = @_;
+##  my ( $self, $name, $size ) = @_;
+    my ( $self, undef, $size ) = @_;	# Name unused
     my $syms = '.';
     my $topo = '';
     my $letter = 'A';
@@ -1514,7 +1405,8 @@ sub _set_latin {
 }
 
 sub _set_null {
-    my ($self, $name, $value) = @_;
+##  my ( $self, $name, $value ) = @_;
+    my ( $self, undef, $value ) = @_;	# Name unused
     my ($size, $columns, $rows) = ref $value ? @$value : split ',', $value;
     $self->{cell} = [];		# The cells themselves.
     $self->{set} = {};		# The sets themselves.
@@ -1532,7 +1424,7 @@ sub _set_null {
 }
 
 sub _set_number {
-    my ($self, $name, $value) = @_;
+    my ( $self, $name, $value ) = @_;
     _looks_like_number ($value) or croak <<eod;
 Error - Attribute $name must be numeric.
 eod
@@ -1541,7 +1433,8 @@ eod
 }
 
 sub _set_quincunx {
-    my ($self, $name, $value) = @_;
+##  my ( $self, $name, $value ) = @_;
+    my ( $self, undef, $value ) = @_;	# Name unused
     my ($order, $gap) = ref $value ? @$value : split ',', $value;
     $order =~ m/\D/ and croak <<eod;
 Error - The quincunx order must be an integer.
@@ -1605,9 +1498,7 @@ eod
 }
 
 sub _set_status_value {
-    my $self = shift;
-    my $name = shift;
-    my $value = shift;
+    my ( $self, $name, $value ) = @_;
     _looks_like_number ($value) or croak <<eod;
 Error - Attribute $name must be numeric.
 eod
@@ -1621,17 +1512,15 @@ eod
 }
 
 sub _set_sudoku {
-    my $self = shift;
-    my $name = shift;
-    my $order = shift;
-    $self->set (brick => [$order, $order, $order * $order]);
+##  my ( $self, $name, $order ) = @_;
+    my ( $self, undef, $order ) = @_;	# Name unused
+    $self->set( brick => [ $order, $order ] );
     return;
 }
 
 sub _set_sudokux {
-    my $self = shift;
-    my $name = shift;
-    my $order = shift;
+##  my ( $self, $name, $order ) = @_;
+    my ( $self, undef, $order ) = @_;	# Name unused
     $self->set (sudoku => $order);
     my $size = $order * $order;
     my $size_minus_1 = $size - 1;
@@ -1642,9 +1531,8 @@ sub _set_sudokux {
 }
 
 sub _set_symbols {
-    my $self = shift;
-    my $name = shift;
-    my $value = shift;
+##  my ( $self, $name, $value ) = @_;
+    my ( $self, undef, $value ) = @_;	# Name unused
     my @lst = split '\s+', $value;
     my %hsh;
     my $inx = 0;
@@ -1669,7 +1557,8 @@ eod
 }
 
 sub _set_topology {
-    my ($self, $name, @args) = @_;
+##  my ( $self, $name, @args ) = @_;
+    my ( $self, undef, @args ) = @_;	# Name unused
     $self->{cell} = [];		# The cells themselves.
     $self->{set} = {};		# The sets themselves.
     $self->{largest_set} = 0;
@@ -1719,7 +1608,7 @@ Status values set:
 =cut
 
 sub solution {
-    my $self = shift;
+    my ( $self ) = @_;
 
     $self->{backtrack_stack} or croak <<eod;
 Error - You cannot call the solution() method unless you have specified
@@ -1780,7 +1669,7 @@ in scalar context, it is the symbol itself.
 =cut
 
 sub steps {
-    my $self = shift;
+    my ( $self ) = @_;
     return wantarray ? (@{$self->{backtrack_stack}}) :
 	defined wantarray ?
 	    $self->_format_constraint (@{$self->{backtrack_stack}}) :
@@ -1796,7 +1685,7 @@ puzzle was loaded.
 =cut
 
 sub unload {
-    my $self = shift;
+    my ( $self ) = @_;
     return $self->_unload ()
 }
 
@@ -1818,7 +1707,7 @@ my %constraint_method = (
 );
 
 sub _constrain {
-    my $self = shift;
+    my ( $self ) = @_;
     my $stack = $self->{backtrack_stack} ||= [];	# May hit this
 							# when initializing.
     my $used = $self->{constraints_used} ||= {};
@@ -1887,7 +1776,7 @@ eod
 #	progress.
 
 sub _constraint_F {
-    my $self = shift;
+    my ( $self ) = @_;
     my @stack;
     my $done = 1;
 
@@ -1939,7 +1828,7 @@ eod
 #	N constraint - the only cell which supplies a necessary value.
 
 sub _constraint_N {
-    my $self = shift;
+    my ( $self ) = @_;
     while (my ($name, $set) = each %{$self->{set}}) {
 	my @suppliers;
 	foreach my $inx (@{$set->{membership}}) {
@@ -1985,7 +1874,7 @@ eod
 #	other can be a row or a column.
 
 sub _constraint_B {
-    my $self = shift;
+    my ( $self ) = @_;
     my $done = 0;
     while (my ($int, $cells) = each %{$self->{intersection}}) {
 	next unless @$cells > 1;
@@ -2064,7 +1953,7 @@ sub _constraint_B {
 #	Angus separates naked and hidden tuples.
 
 sub _constraint_T {
-    my $self = shift;
+    my ( $self ) = @_;
     my @tuple;		# Tuple indices
     my %vacant;		# Empty cells by set. $vacant{$set} = [$cell ...]
     my %contributors;	# Number of cells which can contrib value, by set.
@@ -2209,7 +2098,7 @@ sub _constraint_T {
 # ? constraint - initiate backtracking.
 
 sub _constraint_backtrack {
-    my $self = shift;
+    my ( $self ) = @_;
 ##  --$iterations >= 0 or return $self->_unload ('', SUDOKU_TOO_HARD)
 ##	if defined $iterations;
     my @try;
@@ -2261,12 +2150,9 @@ eod
 #		be removed any number of times.
 
 sub _constraint_remove {
-    my $self = shift;
-    my $min = shift;
+    my ( $self, $min, $max, $removal_ok ) = @_;
     $min and $min = @{$self->{cell}} - $min;
-    my $max = shift;
     $max and $max = @{$self->{cell}} - $max;
-    my $removal_ok = shift;
     $self->{no_more_solutions} and return SUDOKU_NO_SOLUTION;
     my $stack = $self->{backtrack_stack} or return SUDOKU_NO_SOLUTION;
     my $used = $self->{constraints_used} ||= {};
@@ -2359,6 +2245,41 @@ eod
     return SUDOKU_NO_SOLUTION;
 }
 
+#	$self->_deprecation_notice( $name );
+#
+#	This method centralizes deprecation. Deprecation is driven of
+#	the %deprecate hash. Level values are:
+#	    false - no warning
+#	    1 - warn on first use
+#	    2 - warn on each use
+#	    3 - die on each use.
+
+{
+
+    my %deprecate = (
+	brick_third_argument	=> {
+	    message	=> 'Specifying 3 values for set( brick => ... ) is deprecated',
+	    level	=> 0,
+	},
+    );
+
+    sub _deprecation_notice {
+	my ( undef, $name ) = @_;	# Invocant unused
+	my $info = $deprecate{$name}
+	    or return;
+	$info->{level}
+	    or return;
+	$info->{level} >= 3
+	    and croak $info->{message};
+	warnings::enabled( 'deprecated' )
+	    and carp $info->{message};
+	$info->{level} == 1
+	    and $info->{level} = 0;
+	return;
+    }
+
+}
+
 #	_format_constraint formats the given constraint for output.
 
 sub _format_constraint {
@@ -2389,7 +2310,7 @@ sub _format_constraint {
 #	looks_like_number.
 
 sub _looks_like_number {
-    local $_ = shift;
+    ( local $_ ) = @_;
     return 0 if !defined ($_) || ref ($_);
     return 1 if m/^[+-]?\d+$/;
     return 0;
@@ -2409,10 +2330,10 @@ sub _looks_like_number {
 #	does not undo the trial.
 
 sub _try {
-    my $self = shift;
-    my $cell = shift;
+    my ( $self, $cell, $new ) = @_;
     $cell = $self->{cell}[$cell] unless ref $cell;
-    defined (my $new = shift) or _fatal (
+    defined $new
+	or _fatal (
 	"_try called for cell $cell->{index} with new value undefined");
     defined (my $old = $cell->{content}) or _fatal (
 	"_try called with old cell $cell->{index} value undefined");
@@ -2506,38 +2427,9 @@ which is a command-driven interface to this module.
 
 =head1 CLIPBOARD SUPPORT
 
-Clipboard support is highly OS-specific. Here is the story by OS - or,
-really, by the contents of $^O:
-
-=head2 cygwin
-
-Under cygwin, we first try to load the Win32::Clipboard module. If this
-succeeds, we use it. If not, we try to use the xclip program, available
-from L<http://freshmeat.net/project/xclip>.
-
-=head2 darwin
-
-Under Darwin, also known as Mac OS X, we use the pbcopy programs to
-copy text to the clipboard, and pbpaste to retrieve text from the
-clipboard. These programs are supposed to come with Mac OS X. If pbcopy
-or pbpaste (depending on what we are trying to do) is not found, we try
-xclip, under the assumption that you are running Darwin without the Mac
-OS X overlay. The xclip program is available from
-L<http://freshmeat.net/project/xclip>.
-
-=head2 MacOS
-
-Under MacOS (meaning OS 9 or below) we currently have no way to put
-text onto the clipboard.
-
-=head2 MSWin32
-
-Under Windows, we use Win32::Clipboard if available.
-
-=head2 Anything else
-
-Under any other operating system, we try to use the xclip program,
-available from L<http://freshmeat.net/project/xclip>.
+Clipboard support is via the L<Clipboard|Clipboard> module. If this is
+not installed, the C<copy()> and C<paste()> methods will throw
+exceptions.
 
 =head1 BUGS
 
@@ -2621,7 +2513,7 @@ Thomas R. Wyant, III (F<wyant at cpan dot org>)
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2006, 2008, 2011-2015 by Thomas R. Wyant, III
+Copyright (C) 2005-2006, 2008, 2011-2016 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
